//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 一人称カメラコントローラヘッダ
 * @author Junpee
 */

#ifndef FIRST_PERSON_CAMERA_CONTROLLER_H_
#define FIRST_PERSON_CAMERA_CONTROLLER_H_

namespace Lamp{

class Camera;

//------------------------------------------------------------------------------
/**
 * 一人称カメラコントローラ
 */
class FirstPersonCameraController{
public:
	/**
	 * コンストラクタ
	 */
	FirstPersonCameraController();

	/**
	 * デストラクタ
	 */
	virtual ~FirstPersonCameraController();

	/**
	 * カメラの設定
	 * @param camera カメラ
	 */
	virtual void setCamera(Camera* camera){ camera_ = camera; }

	/**
	 * セットアップ
	 * @param nearClip ニアクリップ
	 * @param farClip ファークリップ
	 * @param aspect アスペクト比
	 */
	virtual void setup(float nearClip, float farClip, float aspect);

	/**
	 * ウィンドウプロシージャ
	 * @param windowHandle ウィンドウハンドル
	 * @param message メッセージ
	 * @param wParam wメッセージパラメータ
	 * @param lParam lメッセージパラメータ
	 */
	virtual LRESULT windowProcedure(
		HWND windowHandle, u_int message, WPARAM wParam, LPARAM lParam);

	//--------------------------------------------------------------------------
	/**
	 * カメラ位置の設定
	 * @param position カメラ位置
	 */
	virtual void setCameraPosition(const Vector3& position){
		cameraPosition_ = position;
	}

	/**
	 * カメラ位置の取得
	 * @return カメラ位置
	 */
	virtual const Vector3& getCameraPosition() const{ return cameraPosition_; }

	//--------------------------------------------------------------------------
	/**
	 * カメラ回転値の設定
	 * @param rotation カメラ回転値
	 */
	virtual void setCameraRotation(const Vector3& rotation){
		cameraRotation_ = rotation;
	}

	/**
	 * カメラ回転値の取得
	 * @return カメラ回転値
	 */
	virtual const Vector3& getCameraRotation() const{ return cameraRotation_; }

	//--------------------------------------------------------------------------
	/**
	 * 操作方法文字列の取得
	 * @return 操作方法文字列
	 */
	virtual String getOperationString() const{
		return String(
			"左ドラッグ　カメラの向き変更\n"
			"右ドラッグ　上下左右にカメラを移動\n"
			"Shift+右ドラッグ　前後左右にカメラを移動\n"
			"[ e ]　カメラ前移動\n[ d ]　カメラ後移動\n"
			"[ f ]　カメラ右移動\n[ s ]　カメラ左移動\n"
			"[ r ] カメラ上移動\n[ v ] カメラ下移動\n"
			"マウスホイールでズームイン、ズームアウト\n"
			"Shiftを押しながらキー入力すると1/10の速度でカメラが移動します\n"
		);
	}

	/**
	 * インフォメーション文字列の取得
	 * @return インフォメーション文字列
	 */
	const String& getInformationString() const{ return informationString_; }

private:
	// コピーコンストラクタの隠蔽
	FirstPersonCameraController(const FirstPersonCameraController& copy);

	// 代入コピーの隠蔽
	void operator =(const FirstPersonCameraController& copy);

	// カメラ
	Camera* camera_;
	// カメラ移動値
	Vector3 cameraPosition_;
	// カメラ回転値
	Vector3 cameraRotation_;
	// 画角
	float fovY_;
	// インフォメーション文字列
	String informationString_;

	// 移動に関するマウス感度
	float cameraPositionSensibility_;
	// 回転に関するマウス感度
	float cameraRotationSensibility_;
	// キーボード感度
	float keyboardSensibility_;
	// マウスX位置
	int mouseXPosition_;
	// マウスY位置
	int mouseYPosition_;

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of FIRST_PERSON_CAMERA_CONTROLLER_H_
//------------------------------------------------------------------------------
